# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from rest_framework_tus.models import get_upload_model


class UploadSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_upload_model()
        fields = '__all__'



# -*- coding: utf-8
from django.apps import AppConfig


class RestFrameworkTusConfig(AppConfig):
    name = 'rest_framework_tus'

    # noinspection PyUnresolvedReferences
    def ready(self):
        # Import receivers
        from . import receivers



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from dateutil import relativedelta
from django.conf import settings as django_settings

# Retrieve root settings dict
REST_FRAMEWORK_TUS = getattr(django_settings, 'REST_FRAMEWORK_TUS', {})


# Retrieve settings
TUS_UPLOAD_MODEL = REST_FRAMEWORK_TUS.get('UPLOAD_MODEL', 'rest_framework_tus.Upload')
TUS_UPLOAD_EXPIRES = REST_FRAMEWORK_TUS.get('UPLOAD_EXPIRES', relativedelta.relativedelta(days=1))
TUS_UPLOAD_DIR = REST_FRAMEWORK_TUS.get('UPLOAD_DIR', os.path.join(django_settings.BASE_DIR, 'tmp/uploads/'))
TUS_SAVE_HANDLER_CLASS = \
    REST_FRAMEWORK_TUS.get('SAVE_HANDLER_CLASS', 'rest_framework_tus.storage.DefaultSaveHandler')
TUS_MAX_FILE_SIZE = REST_FRAMEWORK_TUS.get('MAX_FILE_SIZE', 4294967296)  # in bytes



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-09 13:24
from __future__ import unicode_literals

from django.db import migrations, models
import django_fsm
import jsonfield.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Upload',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('guid', models.UUIDField(default=uuid.uuid4, unique=True, verbose_name='GUID')),
                ('state', django_fsm.FSMField(default='initial', max_length=50)),
                ('upload_offset', models.PositiveIntegerField(default=0)),
                ('upload_length', models.IntegerField(default=-1)),
                ('upload_metadata', jsonfield.fields.JSONField()),
                ('filename', models.CharField(blank=True, max_length=255)),
                ('temporary_file_path', models.CharField(max_length=4096, null=True)),
                ('expires', models.DateTimeField(blank=True, null=True)),
                ('uploaded_file', models.FileField(upload_to='uploaded')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from abc import ABCMeta, abstractmethod

from django.core.files import File
from six import with_metaclass

from django.utils.module_loading import import_string

from rest_framework_tus import signals
from .settings import TUS_SAVE_HANDLER_CLASS


class AbstractUploadSaveHandler(with_metaclass(ABCMeta, object)):
    def __init__(self, upload):
        self.upload = upload

    @abstractmethod
    def handle_save(self):
        pass

    def run(self):
        # Trigger state change
        self.upload.start_saving()
        self.upload.save()

        # Initialize saving
        self.handle_save()

    def finish(self):
        # Trigger signal
        signals.saved.send(sender=self.__class__, instance=self)

        # Finish
        self.upload.finish()
        self.upload.save()


class DefaultSaveHandler(AbstractUploadSaveHandler):
    destination_file_field = 'uploaded_file'

    def handle_save(self):
        # Save temporary field to file field
        file_field = getattr(self.upload, self.destination_file_field)
        file_field.save(self.upload.filename, File(open(self.upload.temporary_file_path)))

        # Finish upload
        self.finish()


def get_save_handler(import_path=None):
    return import_string(import_path or TUS_SAVE_HANDLER_CLASS)



# -*- coding: utf-8 -*-
from django.conf.urls import url, include
from rest_framework_tus.views import UploadViewSet

from .routers import TusAPIRouter

router = TusAPIRouter()
router.register(r'files', UploadViewSet, base_name='upload')

urlpatterns = [
    url(r'', include(router.urls, namespace='api'))
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

import copy
import tempfile
import hashlib

from .compat import encode_base64


def encode_base64_to_string(string_value):
    """
    Helper to encode a string or bytes value to a base64 string

    :param six.text_types string_value:
    :return str:
    """
    data = copy.copy(string_value)

    if not isinstance(data, bytes):
        if isinstance(data, str):
            data = data.encode('utf-8')
        else:
            data = str(data).encode('utf-8')

    return encode_base64(data).decode('ascii').rstrip('\n')


def encode_upload_metadata(upload_metadata):
    """
    Encodes upload metadata according to the TUS 1.0.0 spec (http://tus.io/protocols/resumable-upload.html#creation)

    :param dict upload_metadata:
    :return str:
    """
    # Prepare encoded data
    encoded_data = [(key, encode_base64_to_string(value))
                    for (key, value) in sorted(upload_metadata.items(), key=lambda item: item[0])]

    # Encode into string
    return ','.join([' '.join([key, encoded_value]) for key, encoded_value in encoded_data])


def write_bytes_to_file(file_path, offset, bytes, makedirs=False):
    """
    Util to write bytes to a local file at a specific offset

    :param str file_path:
    :param int offset:
    :param six.binary_type bytes:
    :param bool makedirs: Whether or not to create the file_path's directories if they don't exist
    :return int: The amount of bytes written
    """
    if makedirs:
        if not os.path.isdir(os.path.dirname(file_path)):
            os.makedirs(os.path.dirname(file_path))

    fh = None
    try:
        try:
            fh = open(file_path, 'r+b')
        except IOError:
            fh = open(file_path, 'wb')
        fh.seek(offset, os.SEEK_SET)
        fh.write(bytes)
    finally:
        if fh is not None:
            fh.close()

    return len(bytes)


def read_bytes_from_field_file(field_file):
    """
    Returns the bytes read from a FieldFile

    :param ~django.db.models.fields.files.FieldFile field_file:
    :return six.binary_type: bytes read from the given field_file
    """
    try:
        field_file.open()
        result = field_file.read()
    finally:
        field_file.close()
    return result


def read_bytes(path):
    """
    Returns the bytes read from a local file at the given path

    :param str path: The local path to the file to read
    :return six.binary_type: bytes read from the given field_file
    """
    with open(path, 'r+b') as fh:
        result = fh.read()
    return result


def write_chunk_to_temp_file(bytes):
    """
    Write some bytes to a local temporary file and return the path

    :param six.binary_type bytes: The bytes to write
    :return str: The local path to the temporary file that has been written
    """
    fd, chunk_file = tempfile.mkstemp(prefix="tus-upload-chunk-")
    os.close(fd)

    with open(chunk_file, 'wb') as fh:
        fh.write(bytes)

    return chunk_file


def create_checksum(bytes, checksum_algorithm):
    """
    Create a hex-checksum for the given bytes using the given algorithm

    :param six.binary_type bytes: The bytes to create the checksum for
    :param str checksum_algorithm: The algorithm to use (e.g. "md5")
    :return str: The checksum (hex)
    """
    m = hashlib.new(checksum_algorithm)
    m.update(bytes)
    return m.hexdigest()


def create_checksum_header(bytes, checksum_algorithm):
    """
    Creates a hex-checksum header for the given bytes using the given algorithm

    :param six.binary_type bytes: The bytes to create the checksum for
    :param str checksum_algorithm: The algorithm to use (e.g. "md5")
    :return str: The checksum algorithm, followed by the checksum (hex)
    """
    checksum = create_checksum(bytes, checksum_algorithm)
    return '{checksum_algorithm} {checksum}'.format(checksum_algorithm=checksum_algorithm, checksum=checksum)


def checksum_matches(checksum_algorithm, checksum, file_path):
    """
    Checks if the given checksum matches the checksum for the data in the file

    :param str checksum_algorithm: The checksum algorithm to use
    :param str checksum: The original hex-checksum to match against
    :param str file_path: The local path to the file to read the data from
    :return bool: Whether or not the newly calculated checksum matches the given checksum
    """
    bytes = read_bytes(file_path)
    bytes_checksum = create_checksum(bytes, checksum_algorithm)
    return bytes_checksum == checksum



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

try:
    from base64 import encodebytes as encode_base64
except (ImportError, AttributeError):
    from base64 import encodestring as encode_base64

try:
    from base64 import decodebytes as decode_base64
except (ImportError, AttributeError):
    from base64 import decodestring as decode_base64
